'use strict'

const path   = require('path')
const config = require('./config/parameters')

const cleanVariable = function(str) {
  return str.replace(/^-*/, '')
}

const cleanProperty = function(str) {
  if (typeof str === 'string') {
    if (str === 'true')  return true
    if (str === 'false') return false
    if (!isNaN(Number(str))) return Number(str)
  }

  return str
}



class Parser {

  constructor( config ) {
    this.config    = typeof config === 'object' ? config : null
    this._checkers = {}
  }

  new() {
    const parser = new Parser(...arguments)
    parser._checkers = JSON.parse(JSON.stringify(this._checkers))
    parser.config    = JSON.parse(JSON.stringify(this.config))
    return parser
  }

  checker(name, checkFn) {
    if (!name || !checkFn) {
      console.warn('No name or check function found')
      return
    }

    this._checkers[name] = checkFn
  }

  parse( parameters, type ) {
    if (!this.config) {
      console.warn('No configuration found')
      return
    }

    type = type || 'valid'

    if (typeof this[`_${type}Parse`] === 'function') {
      return this[`_${type}Parse`]( parameters )
    }

    return {}
  }

  _validParse( parameters ) {
    const params = this._getKeyValue(parameters.slice(0))

    return Object.assign(
      { __: params },
      this._getValidParameters( params )
    )
  }

  _invalidParse( parameters ) {
    const params = this._getKeyValue(parameters.slice(0))

    return Object.assign(
      params,
      this._getValidParameters( params )
    )
  }

  _softParse( parameters ) {
    return this._getKeyValue(parameters.slice(0))
  }

  _getValidParameters( object ) {
    const result = {}

    let param, value
    for (const key in this.config) {
      param = this.config[key]

      if (!param.type) param.type = 'boolean'

      // Get value from key
      value = object[key]

      // Get value from aliases
      if (value === undefined && param.aliases) {
        for (let i = 0, len = param.aliases.length; i < len; i++) {
          value = object[param.aliases[i]]
          if (value !== undefined) break
        }
      }

      // Check the value
      if (this._checkers[param.type]) {
        if (!this._checkers[param.type]( param, value )) {
          value = undefined
        }
      }

      // Get default value
      if (value === undefined) {
        value = this._getDefaultValue(param)
      }

      // Set result value
      if (value !== undefined) {
        result[key] = value
      }

      param = value = undefined

    }

    return result
  }

  _getDefaultValue( arg ) {
    if (arg.hasOwnProperty('default')) {
      return arg.default
    }
    return undefined
  }

  _getFirstArgument( parameters ) {

    let value = undefined

    if (parameters[0].indexOf('-') !== 0) {
      value = parameters[0]
    }

    return value

  }

  _getKeyValue(parameters) {
    const object = { _: [] }

    if (parameters.length === 0) return object

    const params        = parameters
    const matchVarRegex = /^-{1}\w|^-{2}\w/

    let str, key, value

    for (let i = 0, len = params.length; i <= len; i++) {

      str = params[i] || null

      if (key) {

        if (str === null || str.match(matchVarRegex)) {
          value = true
          i--
        }

        else if (str.match(/^\[$/)) {

          let args      = params.slice(i+1)
          const closure = args.indexOf(']')

          if (closure !== -1) {
            args = args.slice(0, closure)
            i += args.length+1
          } else {
            i += args.length
          }

          value = this._getKeyValue(args)

        }

        else {
          value = str
        }

        object[key] = cleanProperty(value)
        key = value = null
        continue
      }

      if (str && str.match(matchVarRegex)) {
        key = cleanVariable(str)

        if (key.indexOf('=') !== -1) {
          const split = key.split('=')
          key         = split[0]
          value       = split[1]

          object[key] = cleanProperty(value)
          key = value = null
          continue
        }

        if (str.match(/^-[a-zA-Z][0-9]$/)) {
          value = key[1]
          key   = key[0]

          object[key] = cleanProperty(value)
          key = value = null
          continue
        }

        continue
      }

      // Special conditions for tasks arguments
      if (str && str.match(/^--$/) && i > 0 && object._.indexOf(params[i-1]) !== -1) {
        key = cleanVariable(params[i-1])
        continue
      }

      if (typeof str === 'string') {
        object._.push( str )
      }

    }

    return object

  }

}

const ParserSingleton = new Parser(config)

ParserSingleton.checker('boolean', function( parameter, value ) {
  return typeof value === 'boolean'
})

ParserSingleton.checker('value', function( parameter, value ) {
  return value !== null && value !== undefined && value !== '' && typeof value !== 'boolean'
})

ParserSingleton.checker('select', function( parameter, value ) {
  if (parameter.values && parameter.values.indexOf(value) === -1) {
    return false
  }
  return true
})

ParserSingleton.checker('file', function( parameter, value ) {
  let res = false
  let ext = ''

  if (ParserSingleton._checkers['value'](parameter, value)) {
    ext = path.extname(value)
    res = ext.length > 1
  }

  if (parameter.extensions) {
    res = parameter.extensions.indexOf(ext) !== -1
  }

  return res
})

module.exports = ParserSingleton